#include <WiFi.h>
#include <EEPROM.h>
#include "STM32OTAFlasher.h"
#include "http.h"
#include "stm32flash.h"
#include "eepromdata.h"
#include <DNSServer.h>
#include <WiFiClient.h>

#define WIFI_STATE_DISCONNECTED   0
#define WIFI_STATE_CONNECTED      1
#define WIFI_STATE_AP             2

IPAddress staticIP(192, 168, 3, 3); // Die gewünschte statische IP-Adresse
IPAddress gateway(192, 168, 3, 1); // Das Gateway
IPAddress subnet(255, 255, 255, 0); // Die Subnetzmaske
DNSServer dnsServer;

static int                          wifi_state = WIFI_STATE_DISCONNECTED;
static bool                         serial_swapped = false;
static WiFiServer                   telnet_server(23);

//static 
static void telnet_setup (void)
{
    telnet_server.begin();
}

//static 
static void telnet_loop (void)
{
    static WiFiClient telnet_client;

    if (!telnet_client || !telnet_client.connected())
    {
        telnet_client = telnet_server.available();
    }

    if (telnet_client && telnet_client.connected())
    {
        while (telnet_client.available())
        {
            Serial.write(telnet_client.read());
        }

        Serial.flush();

        while (Serial.available())
        {
            telnet_client.write(Serial.read());
        }

        telnet_client.flush();
    }
    else
    {
        while (Serial.available())
        {
            Serial.read();
        }
    }
}

//static 
static void services_start (void)
{
    if (!serial_swapped)
    {
        Serial.flush();
        delay(200);
        Serial.end(); // end old serial
        Serial.begin(115200, SERIAL_8E1); // switch to 8 bit even parity, connect to STM32
        serial_swapped = true;
    }

    telnet_setup();
    http_setup();
}

void
wifi_connect (char * ssid, char * ssidkey, bool do_disconnect)
{
    WiFi.mode(WIFI_STA);
    int cnt;

    if (do_disconnect)
    {
        WiFi.disconnect();
        delay(100);
    }

    wifi_state = WIFI_STATE_DISCONNECTED;
    WiFi.begin(ssid, ssidkey);

    if (!serial_swapped)
    {
        Serial.print("Connecting to ");
        Serial.print(ssid);
        Serial.println(" ...");
    }

    for (cnt = 0; cnt < 20; cnt++) // check 10 seconds if connected
    {
        if (WiFi.status() == WL_CONNECTED)
        {
            break;
        }

        delay(500);
    }

    if (WiFi.status() != WL_CONNECTED)
    {
        if (!serial_swapped)
        {
            Serial.print("WiFi connection to '");
            Serial.print(ssid);
            Serial.println("' not established!");
        }
    }
    else
    {
        if (!serial_swapped)
        {
            Serial.println("WiFi connection established.");
            Serial.print("IP address: ");
            Serial.println(WiFi.localIP());
        }

        services_start();
        wifi_state = WIFI_STATE_CONNECTED;
    }
}

void
wifi_ap (const char * apssid, const char * apssidkey)
{
    WiFi.mode(WIFI_AP);

    if (!serial_swapped)
    {
        Serial.println("disconnecting...");
        Serial.flush();
    }

    WiFi.disconnect();
    wifi_state = WIFI_STATE_DISCONNECTED;
    delay(100);

    if (!serial_swapped)
    {
        Serial.println("starting ap mode...");
        Serial.flush();
    }

    WiFi.softAP(apssid, apssidkey);
    WiFi.softAPConfig(staticIP, gateway, subnet);

    if (!serial_swapped)
    {
        Serial.println("begin server...");
        Serial.flush();
        Serial.print("AP ");
        Serial.println(apssid);
        Serial.print("ipaddress ");
        Serial.println(WiFi.softAPIP());
    }

    wifi_state = WIFI_STATE_AP;
    services_start();
}

void
setup(void)
{
    stm32_flash_setup();
    Serial.begin(115200); // use 115200 Baud
    EEPROM.begin(512); // use 512 bytes as EEPROM
    delay(10);
    Serial.println();
    delay(500);
    Serial.println(0x23);
    Serial.println("FIRMWARE 1.0.0"); // print firmware version

    eeprom_read();

    if (eeprom_flags & EEPROM_FLAG_BOOT_AS_AP)
    {
        wifi_ap(eeprom_ap_ssid, eeprom_ap_ssidkey);
    }
    else
    {
        wifi_connect(eeprom_ssid, eeprom_ssidkey, false);
    }

    dnsServer.start(53, "esp32", WiFi.softAPIP());
}

void
loop()
{
    if (digitalRead(0) == LOW)
    {
        wifi_ap(EEPROM_AP_SSID_CONTENT, EEPROM_AP_SSID_KEY_CONTENT);
    }

    if (wifi_state != WIFI_STATE_DISCONNECTED)
    {
        http_loop();
        telnet_loop();
    }
}
